# Copyright 2020 Chen Bainian
# Licensed under the Apache License, Version 2.0

import argparse

from colcon_core.package_discovery import add_package_discovery_arguments
from colcon_core.plugin_system import satisfies_version
from colcon_core.verb import VerbExtensionPoint
from colcon_ed.edit_tools import edit_target_file
from colcon_ed.edit_tools import FileCompleter
from colcon_ed.edit_tools import find_file
from colcon_ed.edit_tools import get_package_path


class EditVerb(VerbExtensionPoint):
    """Edit a file within a package."""

    def __init__(self):  # noqa: D107
        super().__init__()
        satisfies_version(VerbExtensionPoint.EXTENSION_POINT_VERSION, '^1.0')

    def add_arguments(self, *, parser):  # noqa: D102
        parser.add_argument(
            'package_name', nargs='?', metavar='PKG_NAME',
            type=argument_package_name,
            help='Package name')

        args = parser.add_argument(
            'filename', nargs='?', metavar='FILE_NAME',
            type=argument_package_name,
            help='File name')
        args.completer = FileCompleter(package_name_key='package_name')

        add_package_discovery_arguments(parser)

    def main(self, *, context):  # noqa: D102
        args = context.args
        package_path = get_package_path(args.package_name, args)
        if not package_path:
            return (("Package '{args.package_name}' " +
                     'not found').format_map(locals()))

        file_paths = find_file(package_path, args.filename)
        if not file_paths:
            return (("File '{args.filename}' not found " +
                     'in package {args.package_name}').format_map(locals()))

        # Ask the user to choose when there are more than one file found
        if len(file_paths) > 1:
            msg = 'Multiple files found:'
            for i in range(len(file_paths)):
                msg += '\n ' + str(i + 1) + ') ' + file_paths[i]
            print(msg)
            try:
                index = input('Please enter a number: ')
                index = int(index) - 1
            except KeyboardInterrupt:
                return '\n'
            except ValueError:
                return 'Not an integer'
            if index < 0 or index >= len(file_paths):
                return 'Index out of range'
            path = file_paths[index]
        else:
            path = file_paths[0]

        return edit_target_file(path)


def argument_package_name(value):
    """
    Check if an argument is a valid package name.

    Used as a ``type`` callback in ``add_argument()`` calls.
    Package names starting with a dash must be prefixed with a space to avoid
    collisions with command line arguments.

    :param str value: The command line argument
    :returns: The package name
    :raises argparse.ArgumentTypeError: if the value starts with a dash
    """
    if value.startswith('-'):
        raise argparse.ArgumentTypeError('unrecognized argument: ' + value)
    return value.lstrip()



# Copyright 2020 Chen Bainian
# Licensed under the Apache License, Version 2.0

import os
import subprocess

from colcon_core.package_discovery import discover_packages
from colcon_core.package_identification \
    import get_package_identification_extensions


def get_package_path(package_name, args):
    """
    Get the relative package path.

    Use the `package_discovery` extension to find package path.

    :param str package_name: The name of the package to find path
    :param args: The positional arguments to `add_argument()`
    :returns: The package path or None
    """
    extensions = get_package_identification_extensions()
    descriptors = discover_packages(args, extensions)
    package_path = ''
    for pkg in descriptors:
        if(pkg.name == args.package_name):
            package_path = pkg.path
    return package_path


def find_all_files(package_path):
    """
    Get all the files within the path given.

    Ignore certain files like hidden files and `.pyc` files.

    :param str package_path: The relative path
    :return: a list of relative paths of all the files
    """
    paths = []
    for dirpath, dirnames, filenames in os.walk(package_path):
        # ignore folder starting with .
        dirnames[:] = [d for d in dirnames if not d[0] == '.']

        # ignore hidden files and those ending with .pyc
        filenames = [f for f in filenames
                     if f[0] != '.' and f[-4:] != '.pyc']

        for name in filenames:
            paths.append(os.path.join(dirpath, name))

    return paths


def find_file(package_path, filename):
    """
    Get all the files with the same filename in the path given.

    :param str package_path: The relative path
    :param str filename: The filenames to search for
    :return: a list of relative paths
    """
    paths = []
    all_paths = find_all_files(package_path)
    for path in all_paths:
        if filename == os.path.basename(path):
            paths.append(path)
    return paths


def edit_target_file(path):
    """
    Edit the target file given by the path.

    if the `EDITOR` environment variable is set, a command
    `$EDITOR <filename>` will be called. Otherwise, if `EDITOR`
    is not set or set to empty, `vim` will be use by default

    :param str path: The path of the file to be edited
    :return: process return code
    """
    editor = os.environ.get('EDITOR', 'vim')
    if not editor:
        editor = 'vim'
    process = subprocess.Popen([editor, path],
                               shell=(os.name == 'nt'))
    while process.returncode is None:
        try:
            process.communicate()
        except KeyboardInterrupt:
            pass
    return process.returncode


class FileCompleter:
    """Callable returning a list of file names within a package."""

    def __init__(self, *, package_name_key=None): # noqa D107
        self.package_name_key = package_name_key

    def __call__(self, prefix, parsed_args, **kwargs): # noqa D102
        package_name = parsed_args.package_name
        if not package_name:
            return []
        path = get_package_path(package_name, parsed_args)
        if not path:
            return []

        return [os.path.basename(file_path)
                for file_path in find_all_files(path)]
